package MIDlets;

import java.io.*;
import java.util.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import javax.microedition.rms.*;

/**
 *  MIDlet FetchPage can fetch and display a remote web page.
 **/
public class FetchPage extends MIDlet implements CommandListener {
   
  private Display display;

  // Three on-screen command buttons.
  private Command cancelCommand;
  private Command doneCommand;
  private Command fetchCommand;

  // TextField allows the user to input URL.
  private TextField textField;

  // Record store that stores visited URLs.
  private RecordStore URLStore;

  // If the web page is too long, we need to truncate it 
  // to avoid memory overflow. We set the max fetch length 
  // at 2K bytes.
  private int length = 2048;

  public FetchPage() throws Exception {

    // Initialize display and command objects.
    display = Display.getDisplay(this);
    cancelCommand = new Command("CANCEL", Command.CANCEL, 1);
    doneCommand = new Command("DONE", Command.CANCEL, 1);
    fetchCommand = new Command("FETCH", Command.SCREEN, 1);

    // Accessing the RMS record store by name ensures we use the same
    // record store as the one in MIDlet History.
    URLStore = RecordStore.openRecordStore("CUJTestStore", true);
  }

  /**
   * Ask for URL in the initial screen.
   **/
  public void startApp() {

    // Form with TextField to ask for URL.
    Form form = new Form("Fetch Web Page");
    textField = new TextField("URL:", "",
                              80, TextField.ANY);
    form.append( textField );

    // Two buttons on the form.
    form.addCommand(cancelCommand);
    form.addCommand(fetchCommand);

    // The CommandListener is this MIDlet itself.
    form.setCommandListener( (CommandListener) this);

    // Display the form on the LCD screen.
    display.setCurrent(form);

  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
    // Close the record store before we exit.
    try {
      URLStore.closeRecordStore();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Handle events generated by the Commands in the GUI..
   **/
  public void commandAction(Command command, Displayable screen) {

    if (command == cancelCommand) {
      // Quit this MIDlet if "CANCEL" button is pressed.
      destroyApp(false);
      notifyDestroyed();

    } else if (command == doneCommand) {
      // "DONE" is the same as "CANCEL"
      destroyApp(false);
      notifyDestroyed();

    } else if (command == fetchCommand) {
      // "FETCH" command fetches the remote URL. 

      // Get the target URL from the TextField.
      String url = "http://" + textField.getString();

      // The following message will be over-written by the content 
      // from the fetched page if the operation is successful.
      String pageContent = "not defined";


      try {

        // Open a HTTP connection to the target URL.
        HttpConnection conn = (HttpConnection) Connector.open( url );
        conn.setRequestMethod(HttpConnection.GET);

        // Get the response stream from the server.
        DataInputStream is = conn.openDataInputStream();

        // Read the data from the stream into a String.
        // The reader truncates above "length" to avoid memory overflow.
        StringBuffer pageContentBuff = new StringBuffer();
        int ch, i = 0;
        while ( ((ch = is.read()) != -1) && (i < length) ) {
          pageContentBuff.append( (char)ch );
          i++;
        }
        pageContent = pageContentBuff.toString();

        // Close the connection.
        is.close();
        conn.close();

        // Store successfully fetched URL in history RMS.
        URLStore.addRecord(url.getBytes(), 0, url.length());

      } catch (Exception e) {
        pageContent = "failed";
        e.printStackTrace();
      }

      // Get rid of empty spaces at ends.
      pageContent = pageContent.trim();

      // Result screen display the server response in a TextBox.
      
      TextBox textBox = new TextBox(url, null,
                                    length, TextField.ANY);
      // Display the fetched content.
      textBox.setString( pageContent );

      // Add a button to the textBox screen.
      textBox.addCommand(doneCommand);
      // Event handle is this class itself.
      textBox.setCommandListener( (CommandListener) this);
      // Display the result screen.
      display.setCurrent(textBox);

    } 
  }

}
