package MIDlets;

import java.io.*;
import java.util.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import javax.microedition.rms.*;

/**
 * History MIDlet lists a list of visited URL.
 **/
public class History extends MIDlet implements CommandListener {
   
  private Display display;

  // on-screen command button.
  private Command doneCommand;

  // Record store that stores visited URLs.
  private RecordStore URLStore;

  // If the history is too long, we need to truncate it 
  // to avoid memory overflow. We set the max length 
  // at 2K bytes.
  private int length = 2048;


  public History() throws Exception {

    display = Display.getDisplay(this);
    doneCommand = new Command("DONE", Command.CANCEL, 1);

    // Accessing the RMS record store by name ensures we use the same
    // record store as the one in MIDlet FetchPage.
    URLStore = RecordStore.openRecordStore("CUJTestStore", true);
  }

  /**
   * Setup intial information screen.
   **/
  public void startApp() {

    // This message will be appended by the history records.
    String history = "We have visted the following URLs: \n\n";

    try {

      // Get all records from the RMS record store.
      RecordEnumeration re = URLStore.enumerateRecords(null, null, false);
      while ( re.hasNextElement() ) {
        byte b [] = re.nextRecord();
        history += new String(b) + "\n\n";
      }
    } catch (Exception e) {
      history = "Cannot open database";
      e.printStackTrace();
    }

    // Get rid of empty spaces at ends.
    history = history.trim();

    // If the content to display is too long, we truncate it.
    if ( history.length() > length ) {
      history = history.substring(0, length -1 );
    }
    TextBox textBox = new TextBox("URL History", null,
                                  length, TextField.ANY);
    // Display the history URLs.
    textBox.setString( history );

    // Add a button to the textBox screen.
    textBox.addCommand(doneCommand);
    // Event handle is this class itself.
    textBox.setCommandListener( (CommandListener) this);
    // Display the TextBox screen.
    display.setCurrent(textBox);
  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
    // Close the record store before we exit.
    try {
      URLStore.closeRecordStore();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   *  Event handler. This MIDlet does not generate interesting events.
   **/
  public void commandAction(Command command, Displayable screen) {

    if (command == doneCommand) {
      // Quit this MIDlet if "DONE" button is pressed.
      destroyApp(false);
      notifyDestroyed();
    } 
  }

}
